/*      main.cpp
 *
 * PuTTY engine .exe main program. Creates the engine object and
 * starts the user-interface application.
 *
 * Copyright 2002 Petteri Kangaslampi
 *
 * See license.txt for full copyright and license information.
*/

#include <e32std.h>
#include <e32base.h>
#include <eikdll.h>
#include <apacmdln.h>
#include <stdlib.h>

extern "C" {
#define PUTTY_DO_GLOBALS
#include "putty.h"
}

#include "puttyengineimp.h"

#ifdef __WINS__
_LIT(KPuttyDir, "z:\\system\\apps\\putty\\");
#endif
_LIT(KUiApplication, "puttyui.app");


static void doMainL() {

    // Initialize the engine
    CPuttyEngineImp *engine = CPuttyEngineImp::NewLC();

    // Determine the full path to the UI application
#ifdef __WINS__
    TFileName *appFile = new (ELeave) TFileName;
    *appFile = KPuttyDir;
#else
    TFileName *processFile = new (ELeave) TFileName;
    *processFile = RProcess().FileName();
    TParse *parsa = new (ELeave) TParse;
    parsa->Set(*processFile, NULL, NULL);
    TFileName *appFile = new (ELeave) TFileName;
    *appFile = parsa->DriveAndPath();
#endif
    appFile->Append(KUiApplication);

    // Set command line up -- put the pointer to the engine object into the
    // "tail end" of the command line.
    CApaCommandLine *cl = CApaCommandLine::NewL();
    CleanupStack::PushL(cl);
    cl->SetCommandL(EApaCommandRun);
    cl->SetLibraryNameL(*appFile);

    TBuf8<8> buf;
    _LIT8(KPtrFormat, "%x");
    MPuttyEngine *peng = engine;
    buf.Format(KPtrFormat, (TUint) peng);
    cl->SetTailEndL(buf);

    // Clean up
    delete appFile;
#ifndef __WINS__
    delete parsa;
    delete processFile;
#endif

    CleanupStack::Pop(2); // cl, engine
    // Either the cleanup stack or the heap seems to get messed by the app --
    // maybe it sets its own heap and doesn't restore our old one, and thus we
    // cannot deallocate anything afterwards?

    // And go!
    EikDll::RunAppInsideThread(cl);
}


GLDEF_C TInt E32Main() {
    
    CTrapCleanup *cleanup = CTrapCleanup::New();
    TRAPD(error, doMainL());
    if ( error != KErrNone ) {
        User::Panic(_L("PuttyEngine"), error);
    }
    delete cleanup;
    
    return 0;
}

#if defined(__WINS__)

EXPORT_C TInt WinsMain(TAny *)
{
	E32Main();
	return(KErrNone);
}

GLDEF_C TInt E32Dll(TDllReason)
{
	return(KErrNone);
}
#endif
