/** @file puttyengineimp.h
 *
 * PuTTY engine implementation class
 *
 * Copyright 2002 Petteri Kangaslampi
 *
 * See license.txt for full copyright and license information.
*/

#ifndef __PUTTYENGINEIMP_H__
#define __PUTTYENGINEIMP_H__

#include <e32std.h>
#include <stdarg.h>
#include <gdi.h>
extern "C" {
#include "putty.h"
}
#include "puttyengine.h"
#include "puttyclient.h"
#include "epocnet.h"

/**
 * PuTTY engine implementation class. Takes care of implementing the
 * MPuttyEngine interface on top of the core PuTTY software.
 * @see MPuttyEngine
 */
class CPuttyEngineImp : public CActive, public MPuttyEngine,
                        public MSocketWatcher {

public:
    static CPuttyEngineImp *NewL();
    static CPuttyEngineImp *NewLC();
    ~CPuttyEngineImp();

    // Methods corresponding to PuTTY callbacks
    void putty_fatalbox(char *p, va_list ap);
    void putty_connection_fatal(char *p, va_list ap);
    void putty_do_text(int x, int y, char *text, int len,
                       unsigned long attr, int lattr);
    void putty_verify_ssh_host_key(char *host, int port, char *keytype,
                                   char *keystr, char *fingerprint);
    void putty_askcipher(char *ciphername, int cs);
    void putty_palette_set(int n, int r, int g, int b);
    void putty_palette_reset();
    void putty_do_cursor(int x, int y, char *text, int len, unsigned long attr,
                         int lattr);

    // MPuttyEngine methods
    virtual void Init(MPuttyClient *aClient, const TDesC &aDataPath);
    virtual void Close();
    virtual Config *GetConfig();
    virtual TInt Connect();
    virtual void GetErrorMessage(TDes &aTarget);
    virtual void Disconnect();
    virtual void SetTerminalSize(TInt aWidth, TInt aHeight);
    virtual void RePaintWindow();
    virtual void SendKeypress(TKeyCode aCode, TUint aModifiers);
    virtual void AddRandomNoise(const TDesC8& aNoise);

    // MSocketWatcher methods
    virtual void SocketOpened();
    virtual void SocketClosed();

    // Terminal update callback
    static TInt UpdateTerminal(TAny *anAny);

private:
    CPuttyEngineImp();
    void ConstructL();
    virtual void RunL();
    virtual void DoCancel();

    enum {
        EStateNone = 0,
        EStateInitialized,
        EStateConnected,
        EStateDisconnected,
        EStateFatalConnectionError
    } iState;

    MPuttyClient *iClient;
    char *iConnError;
    char *iFatalConnError;
    TInt iNumSockets;
    TInt iTermWidth, iTermHeight;
    CPeriodic *iTermUpdatePeriodic;

    enum {
        KNumColors = 24
    };

    TRgb iDefaultPalette[KNumColors];
    TRgb iPalette[KNumColors];
};


#endif
