/** @file terminalkeys.h
 *
 * Sends a key to the output, taking terminal emulation settings into account.
 * This really should be a part of the terminal emulation code, and not in the
 * frontend!
 *
 * The keyboard translation code also should use some kind of Putty-internal
 * key codes, instead of system-specific codes, which would make it portable.
 * Now each port has to handle keyboard completely separately.
 *
 * Based on TranslateKey() in window.c.
 *
 * Copyright 2002 Petteri Kangaslampi
 *
 * See license.txt for full copyright and license information.
*/

#ifndef __TERMINALKEYS_H__
#define __TERMINALKEYS_H__

#include <e32std.h>
#include <e32keys.h>

/** 
 * Sends a keypress to the backend. Used in the engine implementation.
 * 
 * @param aCode Key code
 * @param aModifiers Key modifiers (TKeyEvent::iModifiers)
 */
void SendKey(TKeyCode aCode, TUint aModifiers);

#endif
