/*    epocmemory.cpp
 *
 * Symbian OS memory management routines for PuTTY
 *
 * Copyright 2002 Petteri Kangaslampi
 *
 * See license.txt for full copyright and license information.
*/

#include <e32std.h>
#include <e32svr.h>
#include <string.h>
#include <stdlib.h>

extern "C" {
#include "putty.h"
#include "assert.h"
}


static RHeap *heap;


void epoc_memory_init() {
    heap = UserHeap::ChunkHeap(NULL, 64*1024, 1024*1024, 128*1024);
    if ( !heap ) {
        fatalbox("Couldn't create heap -- out of memory?");
    }
}


void epoc_memory_free() {
    heap->Close();
}


void *safemalloc(size_t size)
{
    void *p;
    assert(heap);    
    p = heap->Alloc((TInt)size);
    if ( !p ) {
        fatalbox("Out of memory");
        exit(1);
    }

    return p;
}

void *saferealloc(void *ptr, size_t size)
{
    void *p;
    
    if (!ptr) {
	p = heap->Alloc((TInt)size);
    } else {
        p = heap->ReAlloc(ptr, (TInt) size);
    }
    
    if (!p) {
        fatalbox("Out of memory");
        exit(1);
    }

    return p;
}

void safefree(void *ptr)
{
    if ( ptr ) {
        heap->Free(ptr);
    }
}
