/*    puttystart.cpp
 *
 * PuTTY startup application
 *
 * Copyright 2002 Petteri Kangaslampi
 *
 * See license.txt for full copyright and license information.
*/

#include <e32std.h>
#include <apgcli.h>
#include <apacmdln.h>
#include <apgtask.h>
#include <apgwgnam.h>
#include "puttystart.h"

#ifdef __WINS__
_LIT(KPuttyEngine, "puttyengine.dll");
#else
_LIT(KPuttyEngine, "puttyengine.exe");
#endif

_LIT(KAssert, "puttystart.cpp");
#define assert(x) __ASSERT_ALWAYS(x, User::Panic(KAssert, __LINE__))

// PuTTY application UID
const TUid KUidPutty = { 0x101f9075 };



/*************************
 * Application
 *************************/

const TUid KUidPuttyStart = { 0x101f9074 };

TUid CPuttyStartApplication::AppDllUid() const {
    return KUidPuttyStart;
}

CApaDocument *CPuttyStartApplication::CreateDocumentL() {
    return new (ELeave) CPuttyStartDocument(*this);
}



/*************************
 * Document
 *************************/

CPuttyStartDocument::CPuttyStartDocument(CEikApplication &anApp)
    : CEikDocument(anApp) {
}

CEikAppUi *CPuttyStartDocument::CreateAppUiL() {
    return new (ELeave) CPuttyStartAppUi;
}



/*************************
 * Application UI
 *************************/

CPuttyStartAppUi::CPuttyStartAppUi() {
}

void CPuttyStartAppUi::ConstructL() {
    BaseConstructL();

    // If we have an existing PuTTY session, just raise it to the front and
    // exit. There seem to be problems running multiple copies simultaneously,
    // so at least for the time being we'll take the easy way out and prevent
    // it. (FIXME?)
    if ( RaiseUiL() ) {
        Exit();
        return;
    }

    // Determine the path to the PuTTY engine EXE
    TFileName appDll = iDocument->Application()->AppFullName();
    TParse parsa;
    User::LeaveIfError(parsa.SetNoWild(appDll, NULL, NULL));
    TFileName engineExe = parsa.DriveAndPath();
    engineExe.Append(KPuttyEngine);

    // Connect to launch server
    RApaLsSession ls;
    User::LeaveIfError(ls.Connect());
    CleanupClosePushL(ls);    

    // Build a command line to start the engine and start it
    CApaCommandLine *cl = CApaCommandLine::NewLC();
    cl->SetCommandL(EApaCommandRun);
    cl->SetLibraryNameL(engineExe);
    User::LeaveIfError(ls.StartApp(*cl));

    // Some times the new PuTTY gets thrown to the background, probably due
    // to the weird way we start up. To try to work around this, we'll now try
    // to raise the new UI to the foreground. We'll try a few times, with a
    // delay in between, in case the application doesn't get started quite
    // immediately
    TInt timeout = 20;
    while ( (!RaiseUiL()) && (timeout--) ) {
        User::After(100000);
    }

    // Cleanup and exit
    CleanupStack::PopAndDestroy(2); // cl, ls    
    Exit();
}


CPuttyStartAppUi::~CPuttyStartAppUi() {
}


void CPuttyStartAppUi::HandleCommandL(TInt /*aCommand*/) {
}


// Tries to raise the Putty UI application to front. Returns ETrue if the
// application was found, EFalse if not.
TBool CPuttyStartAppUi::RaiseUiL() {

    RWsSession &ws = CCoeEnv::Static()->WsSession();
    TInt id = 0;
    CApaWindowGroupName::FindByAppUid(KUidPutty, ws, id);
    if ( id == KErrNotFound ) {
        return EFalse;
    }
    TApaTask task(ws);
    task.SetWgId(id);
    task.BringToForeground();
    return ETrue;
}



/*************************
 * View
 *************************/

CPuttyStartAppView::CPuttyStartAppView() {
}

CPuttyStartAppView::~CPuttyStartAppView() {
}

void CPuttyStartAppView::ConstructL(const TRect &aRect) {
    CreateWindowL();
    SetRect(aRect);
    ActivateL();
}


void CPuttyStartAppView::Draw(const TRect & /*aRect*/) const {
    CWindowGc &gc = SystemGc();
    gc.Reset();
    gc.Clear();
}



/*************************
 * Static methods
 *************************/

// Application entry point
EXPORT_C CApaApplication *NewApplication() {
    return new CPuttyStartApplication;
}

// DLL entry point
GLDEF_C TInt E32Dll(TDllReason /*aReason*/) {
    return KErrNone;
}

